from .streamio import get_client, StreamObject
from datetime import date, datetime


class StreamViewSetMixin:
    """
    A mixin for using with DRF viewsets.
    Provides a GET /:resource/:id/stream/ endpoint

    """
    from rest_framework.decorators import action

    @action(detail=True, methods=['GET'])
    def stream(self, request, pk=None):
        from rest_framework.response import Response
        options_fields = ['limit', 'offset', 'id_gt', 'id_gte', 'id_lt', 'id_lte', 'offset', 'ranking', 'enrich']
        extra_args = {}
        for field in options_fields:
            value = request.GET.get(field, None)
            if value is not None:
                extra_args[field] = value
                if field == 'enrich':
                    true_values = ['true', 'True', 't', '1', 1]
                    if value in true_values:
                        extra_args[field] = True
                    else:
                        extra_args[field] = False

        stream = self.get_object().get_feed(**extra_args)
        return Response(stream)

    @action(detail=True, methods=['GET'])
    def streamactivity(self, request, pk=None):
        from rest_framework.response import Response

        feed = self.get_object().get_feed(**{"limit": 100, "enrich": False})

        # from streamio.streamio import get_client
        # cli = get_client()
        # feed = cli.feed("user", 7).get(**{"limit": 100, "enrich": False})

        activities = feed.get('results', [])
        action_breakdown = {}
        timeseries = { "total": {} }
        to_time = None
        from_time = None

        if activities:
            to_time = activities[0].get('time')
            from_time = activities[len(activities) - 1].get('time')

            for activity in activities:
                object_type = activity.get('object').split(":")[1]
                verb = activity.get('verb')
                key = "{}:{}".format(object_type, verb)
                count = action_breakdown.get(key, 0)
                action_breakdown[key] = (count + 1)

                day = activity.get('time').date().isoformat()
                count = timeseries.get("total").get(day, 0)
                key_count = timeseries.get(key, {}).get(day, 0)

                timeseries["total"][day] = (count + 1)
                if not (key in timeseries):
                    timeseries[key] = {}
                timeseries[key][day] = (key_count + 1)


        return Response({
            "period": {
                "from": from_time,
                "to": to_time
            },
            "count": len(activities),
            # stacked timeseries:
            "timeseries": timeseries,
            "radial": action_breakdown
        })

class StreamModelMixin:

    def track_action(self, verb, by = None, create_collection = True, force_update = False, date_field = None):
        """
        # minimal:
        todo.track_action('finish')
        """
        stream = StreamObject(self)
        enriched = None
        if create_collection:
            enriched = stream.enrich(force_update = force_update)

        if by is None:
            by = getattr(self, self.feed_actor_field)

        is_onceoff_action = verb in self.feed_once_off_actions

        custom_message = None
        if getattr(self, 'formatted_feed_message', None) is not None:
            custom_message = self.formatted_feed_message(verb=verb)

        activity = stream.perform_action(
            by,
            verb,
            is_onceoff_action=is_onceoff_action,
            custom_message=custom_message,
            date_field = date_field
        )
        return {
            "object": enriched,
            "activity": activity
        }

    def add_notification(self, verb, message, users_to_notify = None, forward = {}, *args, **kwargs):
        if users_to_notify is None:
            actor_id = getattr(self, self.feed_actor_field, None)
            if actor_id is not None:
                users_to_notify = [actor_id]

        stream = StreamObject(self)
        return stream.add_notification(
            users_to_notify = users_to_notify,
            verb = verb,
            message = message,
            forward = forward
        )

    def get_feed(self, **kwargs):
        options = {
            "enrich": True
        }
        options.update(kwargs)
        client = get_client()
        return client.feed(
            self.feed_name,
            self.pk
        ).get(**options)



import stream, importlib
from stream import exceptions
from django.conf import settings
from django.contrib.auth import get_user_model

from datetime import datetime, date
import os

def is_datetime(dt):
    """
    Returns True if is datetime
    Returns False if is date
    Returns None if it is neither of these things
    """
    try:
        dt.date()
        return True
    except:
        if isinstance(dt, date):
            return False
    return None

def get_client():
    return stream.connect(
        os.environ.get('STREAMIO_KEY', '..'),
        os.environ.get('STREAMIO_SECRET', '..')
    )

def get_serializer_class(serializer_path):
    bits = serializer_path.split('.')
    class_name = bits.pop()
    module_string = (".").join(bits)
    mod = importlib.import_module(module_string)
    return getattr(mod, class_name)

def get_notification_feed_group():
    return os.environ.get('NOTIFICATION_FEED', 'notification')

class StreamObject:
    """
from example_app.models import todo
from streamio.streamio import get_client, StreamObject
item = Todo.objects.all().order_by('?').first()
StreamObject(get_client(), i).perform_action(actor="1", action="test_action")
    """

    def __init__(self, object):
        self.client = get_client()
        self.collection_name = object.collection.lower()
        self.object = object
        self.object_id = str(object.id)

    def generate_foreign_key(self, verb, is_onceoff_action = False, date_field = None):

        if is_onceoff_action == True:
            return "{}:{}".format(self.collection_name, verb)

        timestamp = datetime.now().isoformat()
        if date_field is not None:
            timestamp = getattr(self.object, date_field).isoformat()

        return "{}:{}:{}".format(
            self.collection_name,
            verb,
            timestamp
        )

    def get_to_feeds(self):
        feed_mapping = getattr(self.object, 'feed_related_mapping', None)
        to_mapping = []
        if feed_mapping is not None:
            for feed_slug, id_field in feed_mapping:
                id = getattr(self.object, id_field, None)
                if id is not None:
                    is_list = isinstance(id, list)
                    if is_list:
                        for item in id:
                            to_field = '{}:{}'.format(feed_slug, item)
                            to_mapping.append(to_field)
                    else:
                        to_field = '{}:{}'.format(feed_slug, id)
                        to_mapping.append(to_field)
        return to_mapping

    def get(self):
        return self.client.collections.get(
            self.collection_name,
            self.object_id
        )

    def get_feed(self, enrich=True):
        return self.client.feed(
            self.collection_name,
            self.object_id
        ).get(enrich)

    def add_notification(self, users_to_notify, verb, message, forward, *args, **kwargs):
        feed_group = get_notification_feed_group()
        object_reference = self.client.collections.create_reference(
            self.collection_name,
            self.object_id
        )
        activities = []
        for user_id in users_to_notify:
            notifications = self.client.feed(feed_group, user_id)
            user_reference = self.client.users.create_reference(user_id)
            response = notifications.add_activity({
                "actor": user_reference,
                "verb": "add",
                "object": object_reference,
                "message": message,
                "forward": forward
            })
            activities.append(response)
        return activities

    def perform_action(self, actor_id, verb, time = None, is_onceoff_action = False, date_field = None, custom_message = None):
        actor_id = str(actor_id)
        user_reference = self.client.users.create_reference(actor_id)
        object_reference = self.client.collections.create_reference(
            self.collection_name,
            self.object_id
        )
        activity_id = self.generate_foreign_key(
            verb,
            is_onceoff_action=is_onceoff_action,
            date_field=date_field
        )
        activity_data = {
            "actor": user_reference,
            "verb": verb,
            "object": object_reference,
            "foreign_id": activity_id,
        }
        if custom_message is not None:
            activity_data['message'] = custom_message

        to_feeds = self.get_to_feeds()
        if to_feeds:
            activity_data['to'] = to_feeds

        if time is not None:
            activity_data["time"] = time

        if date_field is not None:
            date_value = getattr(self.object, date_field, None)
            if is_datetime(date_value):
                if time is None:
                    activity_data["time"] = date_value.isoformat()
        feed = self.client.feed("user", actor_id)
        return feed.add_activity(activity_data)

    def enrich(self, serializer = None, force_update = False):

        if serializer is None:
            serializer_class_string = self.object.enrichment_serializer
            serializer = get_serializer_class(serializer_class_string)

        serialized = serializer(self.object).data
        try:
            return self.client.collections.add(
                self.collection_name,
                serialized,
                id=self.object_id
            )
        except exceptions.StreamApiException as e:
            if force_update:
                return self.client.collections.update(
                    self.collection_name,
                    self.object_id,
                    serialized
                )
            else:
                return self.get()

class StreamUser:
    """
    """

    def __init__(self, user_id):
        self.client = get_client()
        self.user_id = str(user_id)

    def get_or_create(data = {}):
        # data = {"name": "Jack", "profile_picture": "https://goo.gl/XSLLTA"}
        user = self.client.users.add(
            self.user_id,
            data,
            get_or_create=True
        )

    def perform_action(self, object, verb, time = None):
        actor_id = self.user_id
        user_reference = self.client.users.create_reference(actor_id)
        object_reference = self.client.collections.create_reference(
            object.collection,
            str(object.id)
        )
        activity_data = {
            "actor": user_reference,
            "verb": verb,
            "object": object_reference,
        }
        if time is not None:
            activity_data["time"] = time

        feed = self.client.feed("user", actor_id)
        return feed.add_activity(activity_data)

    def get_feed(self, enrich=True):
        feed = self.client.feed("user", self.user_id)
        return feed.get(enrich=enrich)

    def get_notifications(self, enrich=True):
        feed_group = get_notification_feed_group()
        feed = self.client.feed(feed_group, self.user_id)
        return feed.get(enrich=enrich)

    def enrich(self, data, force_update = False):
        return self.get_or_create_user(data)
        # todo: upsert



import setuptools

with open("./README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="dj-streamio",
    version="0.2.1",
    author="Christo Crampton",
    author_email="info@38.co.za",
    description="",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/schoolorchestration/libs/dj-streamio",
    # packages=setuptools.find_packages(),
    packages=['streamio'],
    install_requires=['stream-python'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)